#include "config.h"
#include "src/domain/domainfacade.h"

#ifdef QTGUI
#include "presentation/frontends/qtfrontend/qtfrontend.h"
#endif

#include "presentation/frontends/nonguifrontend/nonguifrontend.h"

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stddef.h>
#include <dirent.h>
#include <stdio.h>
#include <libgen.h>


void init();
void cleanup();


int main(int argc, char **argv) 
{
	init();
	atexit(cleanup);
	
	// returns a pointer that is allocated with new
	DomainFacade *facadePtr = DomainFacade::getFacade();
	int ret = 0;

	// if program is started with --nonGUI
	if ( argc > 1 && strcmp(argv[1], "--nonGUI") == 0 ) {
		NonGUIFrontend nonGUIFrontend(facadePtr);
		facadePtr->registerFrontend(&nonGUIFrontend);
		ret = nonGUIFrontend.run(argc, argv); 
	}
	else {
#ifdef QTGUI
		QtFrontend *qtFrontend = new QtFrontend(argc, argv);
		facadePtr->registerFrontend(qtFrontend);
		ret = qtFrontend->run(argc, argv);
		delete qtFrontend;
		qtFrontend = NULL;
#endif
	}
	
	delete facadePtr;
	facadePtr = NULL;
	
	return ret;
}


// creates directories if they already not exist.
void init()
{
	// in case the the application exited abnormally and
	// couldn't run cleanup ... doh! it wouldn't happen
	// anyway :)
	cleanup();
	
	char tmp[256] = {0};
	sprintf( tmp, "%s/.stopmotion/", getenv("HOME") );
	
	// is ~/.stopmotion doesn't exist
	if (access(tmp, F_OK) == -1) {
		mkdir(tmp, 0755);
	}
	
	sprintf( tmp, "%s/.stopmotion/tmp/", getenv("HOME") );
	mkdir(tmp, 0755);
	sprintf( tmp, "%s/.stopmotion/trash/", getenv("HOME") );
	mkdir(tmp, 0755);
	sprintf( tmp, "%s/.stopmotion/packer/", getenv("HOME") );
	mkdir(tmp, 0755);
}


// cleanup the directories
void cleanup()
{
	char command[256] = {0};
	
	sprintf( command, "rm -rf %s/.stopmotion/tmp/", getenv("HOME") );
	system(command);
	sprintf( command, "rm -rf %s/.stopmotion/trash/", getenv("HOME") );
	system(command);
	sprintf( command, "rm -rf %s/.stopmotion/packer/", getenv("HOME") );
	system(command);
}
