/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// This widget is created from a widget in an example program for Qt. Available at
// http://www.libsdl.org/cvs/qtSDL.tar.gz

#include "frameview.h"
#include "src/domain/domainfacade.h"
#include "src/technical/grabber/commandlinegrabber.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <qsize.h>
#if defined(Q_WS_X11)
#include <X11/Xlib.h>
#endif
#include <iostream>
#include <qmessagebox.h>


FrameView::FrameView(QWidget *parent, const char *name)
 : VideoView(parent, name, WNoAutoErase)
{
	Logger::get().logDebug("FrameView is attatched to the model"
		" and the model to FrameView");
	
	screen = NULL;
	currImage = NULL;
	widthConst  = 4;
	heightConst = 3;
	isPlayingVideo = false;
	
	grabber = NULL;
	
	atexit(SDL_Quit);
	setNormalRatio();
}


FrameView::~FrameView()
{
	SDL_FreeSurface(currImage);
	SDL_FreeSurface(screen);
	currImage = NULL;
	screen = NULL;
}


void FrameView::setWidescreenRatio()
{
	widthConst  = 16;
	heightConst = 9;
	float minWidth = 300 * (16 / 9);
	this->setMinimumSize( (int)minWidth, 300);
	this->update();
}


void FrameView::setNormalRatio()
{
	widthConst  = 4;
	heightConst = 3;
	this->setMinimumSize(400, 300);
	this->update();
}


// abstract functions that must be implemented, but who aren't needed in
//this particular widget.
void FrameView::updateAdd(const vector<char*>&, unsigned int, Frontend*) {}
void FrameView::updateRemove(unsigned int, unsigned int) {}
void FrameView::updateMove(unsigned int, unsigned int, unsigned int) {}


void FrameView::updateNewActiveFrame(int frameNumber)
{
	if (frameNumber > -1) {
		Frame *f = DomainFacade::getFacade()->getFrame(frameNumber);
		const char *str = f->getImagePath();
		setActiveFrame(str);
	}
	else {
		SDL_FreeSurface(currImage);
		currImage = NULL;
		this->repaint();
	}
}


void FrameView::updatePlayFrame(int frameNumber)
{
	if (frameNumber > -1) {
		Frame *f = DomainFacade::getFacade()->getFrame(frameNumber);
		const char *str = f->getImagePath();		
		setActiveFrame(str);
	}
	else {
		SDL_FreeSurface(currImage);
		currImage = NULL;
		this->repaint();
	}
}


void FrameView::resizeEvent(QResizeEvent*)
{	
	char variable[64];
	// We could get a resize event at any time, so clean previous mode
	screen = NULL;
	SDL_QuitSubSystem(SDL_INIT_VIDEO);

	// Set the new video mode with the new window size
	sprintf(variable, "SDL_WINDOWID=0x%lx", winId());
	putenv(variable);
	if ( SDL_InitSubSystem(SDL_INIT_VIDEO) < 0 ) {
		char msg[80] = "Unable to init SDL: ";
		strcat( msg, SDL_GetError() );
		Logger::get().logFatal(msg);
    }
	
	screen = SDL_SetVideoMode(width(), height(), 0, 0);
	if (!screen) {
		char msg[80] = "Unable to set vide mode: ";
		strcat( msg, SDL_GetError() );
		Logger::get().logFatal(msg);
	}
}


void FrameView::paintEvent(QPaintEvent *)
{
#if defined(Q_WS_X11)
	// Make sure we're not conflicting with drawing from the Qt library
	XSync(QPaintDevice::x11Display(), FALSE);
#endif
	if (screen) {
		SDL_FillRect(screen, NULL, 0);
		
		if ( currImage ) {
			SDL_Rect dst;
			dst.x = (screen->w - currImage->w) / 2;
			dst.y = (screen->h - currImage->h) / 2;
			dst.w = currImage->w;
			dst.h = currImage->h;
			SDL_BlitSurface(currImage, NULL, screen, &dst);

			if(isPlayingVideo) {
				SDL_Surface mixingImage;
				
				int activeFrame = DomainFacade::getFacade()->getActiveFrameNumber();
				for(int i = 1; (i<=mixCount) && (activeFrame-i >= -1); i++) {
					mixingImage = *IMG_Load( DomainFacade::getFacade()->
							getFrame(activeFrame+1-i)->getImagePath() );
					SDL_Rect dst2;
					dst2.x = (screen->w - mixingImage.w) / 2;
					dst2.y = (screen->h - mixingImage.h) / 2;
					dst2.w = mixingImage.w;
					dst2.h = mixingImage.h;
					
					SDL_SetAlpha(&mixingImage, SDL_SRCALPHA, 80/i);
					
					SDL_BlitSurface(&mixingImage, NULL , screen, &dst2);
				}
			
			
				/*
				activeFrame = DomainFacade::getFacade()->getActiveFrameNumber();
				if( activeFrame >=0 ) {
					SDL_Surface mixingImage = *IMG_Load( DomainFacade::getFacade()->
							getFrame(activeFrame)->getImagePath() );
					SDL_Rect dst2;
					dst2.x = (screen->w - mixingImage.w) / 2;
					dst2.y = (screen->h - mixingImage.h) / 2;
					dst2.w = mixingImage.w;
					dst2.h = mixingImage.h;
				
					SDL_SetAlpha(&mixingImage, SDL_SRCALPHA, 100);
					SDL_BlitSurface(&mixingImage, NULL , screen, &dst2);
				}*/
			}
		}
		SDL_Flip(screen);
	}
}


void FrameView::setActiveFrame(const char *fileName)
{
	Logger::get().logDebug("Setting new active frame in FrameView");
	
	if (currImage) {
		SDL_FreeSurface(currImage);
	}
	Logger::get().logDebug("Loading image");
	currImage = IMG_Load(fileName);
	Logger::get().logDebug("Loading image finished");
	this->update();
}


void FrameView::updateClear()
{
	SDL_FreeSurface(currImage);
	currImage = NULL;
	this->update();
}


void FrameView::updateNewScene(int)
{

}


void FrameView::updateRemoveScene(int)
{
}


void FrameView::updateNewActiveScene(int, vector<char*>, Frontend*)
{
}


void FrameView::updateAnimationChanged(vector<Frame*>&)
{

}


void FrameView::updateMoveScene( int, int )
{

}


bool FrameView::on()
{
	this->isPlayingVideo = true;
	
	char tmp[256];
	snprintf(tmp, 256, "%s/.stopmotion/capturedfile.png", getenv("HOME"));
	
	this->grabber = new CommandLineGrabber(tmp);
	if ( grabber->init() ) {
		this->initCompleted();
		grabThread = new ImageGrabThread(this, grabber);
		grabThread->start();
		grabThread->wait(100);

		if (grabThread->wasGrabbingSuccess() == false) {
			QMessageBox::warning(this, tr("Warning"), tr(
					"Grabbing failed. This may happen if you try\n"
					"to grab from an invalid device. Please check\n"
					"your grabber settings in the preferences menu."),
					QMessageBox::Ok,
					QMessageBox::NoButton, 
					QMessageBox::NoButton);
			return false;
		}
	}
	else {
		QMessageBox::warning(this, tr("Warning"), tr(
				"You have to define an image grabber to use.\n"
				"This can be setted in the preferences menu."),
				QMessageBox::Ok,
				QMessageBox::NoButton, 
				QMessageBox::NoButton);
		return false;
	}
	return true;
}


void FrameView::off()
{
	grabThread->terminate();
	grabThread->wait();
	
	delete grabber;
	grabber = NULL;	
	this->isPlayingVideo = false;
	
	SDL_FreeSurface(currImage);
	currImage = NULL;
	this->update();
}


void FrameView::redraw()
{
	char temp[256];
	sprintf( temp, "%s/.stopmotion/capturedfile.png", getenv("HOME") );
	currImage = IMG_Load(temp);
	this->update();
}


/**
 * @todo this function should capture frames.....
 */
void FrameView::capture()
{
}


