/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef FRAMEVIEW_H
#define FRAMEVIEW_H

#include "src/config.h"

#include "videoview.h"
#include "src/technical/grabber/imagegrabber.h"
#include "src/presentation/observer.h"
#include "src/domain/animation/frame.h"
#include "imagegrabthread.h"

#include <vector>
#include <qwidget.h>


class ImageGrabThread;
struct SDL_Surface;

/**
 * Widget for viewing the frames in the animation using SDL. This widget also 
 * serves as videoview widget for displaying video from an external source
 * by grabbing through the harddrive.
 *
 * The reason for this dual responsability is that the system crash when 
 * having two sdl widgets, and this was the easiest/fastest way to work around it.
 *
 * @author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
 */
class FrameView : public VideoView
{
	Q_OBJECT
public:
    FrameView(QWidget *parent=0, const char *name=0);
    ~FrameView();
	
	/**
	 * Sets the view to 4:3 format.
	 */
	void setWidescreenRatio();
	
	/**
	 * Sets the view to 16:9 format.
	 */
	void setNormalRatio();
	
	/**
	 *Function to recieven notification when a frame is added.
	 *
	 */
	void updateAdd(const vector<char*>& frames, unsigned int, Frontend*);
	
	/**
	 *Function to recieve notification when one or more frames are deleted.
	 *@param frameNumber the frame to remove
	 */
	void updateRemove(unsigned int, unsigned int);
	
	/**
	 *Function to recieve notification when one or more frames are moved.
	 *
	 */
	void updateMove(unsigned int fromFrame, unsigned int toFrame, unsigned int movePosition);
	
	
	/**
	 *Function to recieve notification when a new frame is selected.
	 */
	void updateNewActiveFrame(int frameNumber);
	
	/**
	 * Function to receive notification when the model is erased.
	 */
	void updateClear();
	
	/**
	 * Function to recieve notification when a frame is to be played.
	 * @param frameNumber the frame to be played
	 */
	void updatePlayFrame(int frameNumber);
	
	/**
	 * Function to recieve notification when a new scene is added to the
	 * model.
	 * @param index the index of the new scene.
	 */
	void updateNewScene(int index);
	
	/**
	 * Function to recueve notification when a scene is removed from
	 * the model.
	 * @param sceneNumber the scene which has been removed from the model.
	 */
	void updateRemoveScene(int sceneNumber);
	
	/**
	 * Function which recieve notification when a scene in the animation
	 * has been moved.
	 * @param sceneNumber the scene which have been moved.
	 * @param movePosition the position the scene has been moved to.
	 */
	void updateMoveScene(int sceneNumber, int movePosition);
	
	void updateNewActiveScene(int sceneNumber, vector<char*> frames,
		Frontend *frontend);
		
	void updateAnimationChanged(vector<Frame*>& frames);
	
	bool on();
	void off();
	void capture();
	void redraw();
	
protected:
	void resizeEvent(QResizeEvent *);
	void paintEvent(QPaintEvent *);
	
private:
	SDL_Surface *screen;
	SDL_Surface *currImage;
	int widthConst, heightConst;
	bool isPlayingVideo;
	
	ImageGrabThread *grabThread;
	ImageGrabber *grabber;
	
	void setActiveFrame(const char *fileName);

};

#endif
