/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mainwindowgui.h"

#include "src/domain/domainfacade.h"
#include "src/application/soundhandler.h"
#include "videoview.h"
// #include "gstvideoview.h"
#include "flexiblelineedit.h"
#include "licence.h"
#include "preferencesmenu.h"
#include "toolsmenu.h"

#include "graphics/icons/windowicon.xpm"

#include <stdlib.h>
#include <qstatusbar.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qwizard.h>
#include <qevent.h>
#include <qdragobject.h> 
#include <qmessagebox.h>
#include <qapplication.h>
#include <qwhatsthis.h>
#include <qiconset.h>
#include <qfiledialog.h>
#include <qlayout.h>
#include <qtabdialog.h>
#include <qtextedit.h>


#include <iostream>


MainWindowGUI::MainWindowGUI() : QMainWindow( 0, "stopmotion", WDestructiveClose )
{
	centerWidget        = NULL;
	workArea            = NULL;
	frameViewStack      = NULL;
	frameView           = NULL;
	frameBar            = NULL;
	fileMenu            = NULL;
	editMenu            = NULL;
	toolsMenu           = NULL;
	preferencesMenu     = NULL;
	gotoMenu            = NULL;
	helpMenu            = NULL;
	lastVisitedDir      = NULL;
	frameNumberDisplay  = NULL;
	gotoSpinner         = NULL;
	cameraHandler       = NULL;
	editMenuHandler     = NULL;
	runAnimationHandler = NULL;
	modelHandler        = NULL;
	languageHandler     = NULL;
	
	lastVisitedDir = new char[256];
	strcpy( lastVisitedDir, getenv("PWD") );
	centerWidget = new QVBox(this);

	//Initializes and sets up the framebar
	makeFrameBar(centerWidget);
	makePreferencesMenu(centerWidget);
	
	//Initializes and sets up the workarea consisting of a menu and the frameview.
	workArea = new QHBox(centerWidget);
	
	languageHandler = new LanguageHandler( this, this->statusBar() );
	cameraHandler = new CameraHandler( this, this->statusBar() );
	runAnimationHandler = new RunAnimationHandler( this, this->statusBar() );
	modelHandler = new ModelHandler( this, this->statusBar(), frameBar, lastVisitedDir );
	QObject::connect( modelHandler, SIGNAL(modelChanged()), 
			this, SLOT(activateMenuOptions()) );
	editMenuHandler = new EditMenuHandler( this, this->statusBar(), frameBar );
	
	QObject::connect( editMenuHandler, SIGNAL(addFrames(const QStringList &)),
			modelHandler, SLOT(addFrames(const QStringList &)) );
	
	makeToolsMenu(workArea);
	
	makeViews(workArea);
	
	makeGotoMenu(centerWidget);
	setCentralWidget(centerWidget);
	makeStatusBar();
	
	//Initializes and sets up the menues.
	makeMenues();
	
	//This slot will activate/deactivate menu options based on the changes in the model.
	QObject::connect( frameBar, SIGNAL( modelSizeChanged(int) ),
			this, SLOT( modelSizeChanged(int) ) );
	
	this->setIcon( QPixmap(windowicon) );
	this->setAcceptDrops(true);
	this->statusBar()->message( tr("Ready to rumble ;-)"), 2000 );
	this->statusBar()->setSizeGripEnabled(false);
}


MainWindowGUI::~MainWindowGUI()
{
	delete [] lastVisitedDir;
}


void MainWindowGUI::makeMenues()
{
	fileMenu = new QPopupMenu(this);
	int id = fileMenu->insertItem( tr("&New"), this, SLOT(newProject()), CTRL+Key_N );
	QString infoText =
		tr("<h4>New</h4> "
		"<p>Creates a <em>new</em> empty project.</p>");
	fileMenu->setWhatsThis(id, infoText);
	
	id = fileMenu->insertItem( tr("&Open"), this,SLOT(openProject()), CTRL+Key_O );
	infoText =
		tr("<h4>Open</h4> "
		"<p><em>Opens</em> a Stopmotion project file.</p>");
	fileMenu->setWhatsThis(id, infoText);
		
	fileMenu->insertSeparator();
	
	id = fileMenu->insertItem( tr("&Save"), this, SLOT(saveProject()), CTRL+Key_S, SAVE);
	infoText =
		tr("<h4>Save</h4> "
		"<p><em>Saves</em> the current animation project as a Stopmotion .sto file. "
		"<BR>If this project has been saved before it will automaticly be saved to the "
		"previously selected file.</p>");
	fileMenu->setWhatsThis(id, infoText);
	fileMenu->setItemEnabled(SAVE, false);
	
	id = fileMenu->insertItem( tr("Save &As"), this, SLOT(saveProjectAs()), 
		CTRL+SHIFT+Key_S, SAVE_AS);
	infoText =
		tr("<h4>Save As</h4> "
		"<p><em>Saves</em> the current animation project as a Stopmotion .sto file.</p>");
	fileMenu->setWhatsThis(id, infoText);
	fileMenu->setItemEnabled(SAVE_AS, false);

	fileMenu->insertSeparator();
	
	id = fileMenu->insertItem( tr("Add &Frames"), modelHandler, 
			SLOT(chooseFrame()), CTRL+Key_F );
	infoText =
		tr("<h4>Add Frames</h4> "
		"<p><em>Adds</em> one or more frames to the animation.</p>");
	fileMenu->setWhatsThis(id, infoText);
	
	fileMenu->insertSeparator();
	
	id = fileMenu->insertItem( tr("&Quit"), qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );
		infoText =
		tr("<h4>Quit</h4> "
		"<p><em>Quits</em> the program.</p>");
	fileMenu->setWhatsThis(id, infoText);
	
	menuBar()->insertItem( tr("&File"), fileMenu);
	
	
	editMenu = new QPopupMenu(this);
	id = editMenu->insertItem( tr("&Undo"), editMenuHandler, SLOT(undo()), CTRL+Key_Z, UNDO );
	infoText =
		tr("<h4>Undo</h4> "
		"<p><em>Undoes</em> your last operation. You can press undo "
		"several time to undo earlier operations.</p>");
	editMenu->setWhatsThis(id, infoText);
	editMenu->setItemEnabled(UNDO, false);
	
	id = editMenu->insertItem( tr("Re&do"), editMenuHandler, SLOT(redo()), CTRL+SHIFT+Key_Z, REDO );
	infoText =
		tr("<h4>Redo</h4> "
		"<p><em>Redoes</em> your last operation. You can press redo "
		"several times to redo several operations.</p>");
	editMenu->setWhatsThis(id, infoText);
	editMenu->setItemEnabled(REDO, false);
	
	editMenu->insertSeparator();
	
	id = editMenu->insertItem( tr("Cu&t"), CUT, CTRL+Key_X );
	infoText =
		tr("<h4>Cut</h4> "
		"<p><em>Cuts</em> the selected frames out of the animation and adds them "
		"to the clipboard so that you can paste them in somewhere else.</p>");
	editMenu->setWhatsThis(id, infoText);
	editMenu->setItemEnabled(CUT, false);
	
	id = editMenu->insertItem( tr("&Copy"), editMenuHandler, SLOT(copy()), CTRL+Key_C, COPY );
	infoText =
		tr("<h4>Copy</h4> "
		"<p><em>Copies</em> the selected frames to the clipboard. You can "
		"then paste them in another place.</p>");
	editMenu->setWhatsThis(id, infoText);
	editMenu->setItemEnabled(COPY, false);
	
	id = editMenu->insertItem( tr("&Paste"), editMenuHandler, SLOT(paste()), CTRL+Key_V, PASTE );
	infoText =
		tr("<h4>Paste</h4> "
		"<p><em>Pastes</em> the frames which are currently in the clipboard "
		"into the selected location.</p> <p>You can copy/cut images from another "
		"programs and then use this option to paste them into this animation.</p>");
	editMenu->setWhatsThis(id, infoText);
	
	editMenu->insertSeparator();
	
	id = editMenu->insertItem( tr("&Go to frame"), gotoMenu, SLOT(open()), 
			CTRL+Key_G, GOTO );
	infoText =
		tr("<h4>Go to frame</h4> "
		"<p>This will bring up a popup-menu at the bottom where you can choose a "
		"frame you want to <em>go to</em>.</p>");
	editMenu->setWhatsThis(id, infoText);
	editMenu->setItemEnabled(GOTO, false);
		
	id = menuBar()->insertItem( tr("&Edit"), editMenu);
	
	
	QPopupMenu *settingsMenu = new QPopupMenu(this);
	
	id = settingsMenu->insertItem( tr("Disable sound") );
	settingsMenu->setItemEnabled(id, false);
	
	
	//id = settingsMenu->insertItem( tr("&Languages"), CTRL+Key_L);
	QPopupMenu *languagesMenu = new QPopupMenu(settingsMenu);
	
	id = languagesMenu->insertItem( tr("&English"), 
			languageHandler, SLOT(installEnglish()) );
	infoText =
		tr("<h4>English</h4> "
		"<p>Select this menu options to get the <em>English</em> translation of "
		"the application.</p>");
	languagesMenu->setWhatsThis(id, infoText);
	id = languagesMenu->insertItem( tr("&Norwegian"), 
			languageHandler, SLOT(installNorwegian()) );
	infoText =
		tr("<h4>Norwegian</h4> "
		"<p>Select this menu options to get the <em>Norwegian</em> translation of "
		"the application.</p>");
	languagesMenu->setWhatsThis(id, infoText);
	
	id = settingsMenu->insertItem( tr("&Languages"), languagesMenu);
	
	
	settingsMenu->insertSeparator();
	
	
	id = settingsMenu->insertItem( tr("Prefere&nces"), this, SLOT(showPreferencesMenu()), 
			CTRL+Key_P);
	infoText =
		tr("<h4>Preferences</h4> "
		"<p>This will opens a window where you can set <em>preferences</em> "
		"for the application and the current project.</p>");
	settingsMenu->setWhatsThis(id, infoText);
	
	menuBar()->insertItem( tr("&Settings"), settingsMenu);
	
	
	
	QPopupMenu *helpMenu = new QPopupMenu(this);
	id = helpMenu->insertItem(tr("What's &This"), this, SLOT(whatsThis()), SHIFT+Key_F1 );
	infoText =
		tr("<h4>What's This</h4> "
		"<p>This will give you a WhatsThis mouse cursor which can be used to bring up "
		"helpful information like this.</p>");
	helpMenu->setWhatsThis(id, infoText);
	
	helpMenu->insertSeparator();
	
	id = helpMenu->insertItem( tr("&About"), this, SLOT(showAboutDialog()) );
	infoText =
		tr("<h4>About</h4> "
		"<p>This will display a small information box where you can read "
		"general information as well as the names of the developers "
		"behind this excellent piece of software.</p>");
	helpMenu->setWhatsThis(id, infoText);
	
	menuBar()->insertItem( tr("&Help"), helpMenu);
}


void MainWindowGUI::makeFrameBar(QWidget * parent)
{
	frameBar = new FrameBar(parent);
	QString infoText = 
			tr("<h4>FrameBar</h4> "
			"<p>In this area you can see the frames and scenes "
			"in the animations and build the animation "
			"by moving the them around.</p><p>You can "
			"switch to the next and the previous frame "
			"using the <b>arrow buttons</b> or <b>x</b> "
			"and <b>z</b></p> ");
	QWhatsThis::add( frameBar, infoText );
}

  
void MainWindowGUI::makeToolsMenu(QWidget * parent)
{
	toolsMenu = new ToolsMenu(runAnimationHandler, modelHandler, 
			cameraHandler, parent);
	QString infoText = 
			tr("<h4>Tool menu</h4> "
			"<p>This is the tool menu where most of the buttons and widgets "
			"you will need when working on stop motion animations are located.</p>");
	QWhatsThis::add( toolsMenu, infoText );
}


void MainWindowGUI::makePreferencesMenu( QWidget * parent )
{
	preferencesMenu = new FramePreferencesMenu( parent);
	preferencesMenu->setFrameStyle(QFrame::PopupPanel | QFrame::Sunken);
	
	QGridLayout *grid = new QGridLayout( preferencesMenu, 1, 1, 3 );
	
	QString infoText = 
			tr("<h4>Frame preferences menu</h4> "
			"<p>In this menu you can set preferences for the "
			"selected frame/frames, such as <b>subtitles</b>, "
			"<b>sound effects</b>, etc.</p>");
	QWhatsThis::add( preferencesMenu, infoText );
	
	SoundHandler *soundHandler = new SoundHandler( this, this->statusBar(), 
			this->lastVisitedDir );
	QObject::connect( soundHandler, SIGNAL( soundsChanged() ),
			frameBar, SLOT( frameSoundsChanged() ) );
			
	QPushButton *addSoundButton = new QPushButton( tr("Add &sound"), preferencesMenu );
	addSoundButton->setFocusPolicy( QWidget::NoFocus );
	grid->addWidget( addSoundButton, 1, 0 );
	infoText = 
			tr("<h4>Add sound</h4> "
			"<p>With this button you can <em>add sounds</em> to the selected frame.</p> "
			"<p>The sound will begin playing when this frame is shown and play "
			"until it is done.</p>");
	QWhatsThis::add( addSoundButton, infoText );
	QObject::connect( addSoundButton, SIGNAL( clicked() ), 
			soundHandler, SLOT( addSound() ) );
	
	QPushButton *removeSoundsButton = new QPushButton( tr("Remove Sound"),  
			preferencesMenu );
	removeSoundsButton->setFocusPolicy( QWidget::NoFocus );
	grid->addWidget( removeSoundsButton, 2, 0 );
	infoText = 
			tr("<h4>Remove sound</h4> "
			"<p>With this button you can <em>remove</em> the selected sound from "
			"this frame.</p>");
	QWhatsThis::add( removeSoundsButton, infoText );
	QObject::connect( removeSoundsButton, SIGNAL( clicked() ), 
			soundHandler, SLOT( removeSound() ) );
			
	QPushButton *changeNameButton = new QPushButton( tr("Change name"),
			preferencesMenu );
	changeNameButton->setFocusPolicy( QWidget::NoFocus );
	grid->addWidget( changeNameButton, 3, 0 );
	infoText = 
			tr("<h4>Change name</h4> "
			"<p>With this button you can change the name of the selected sound. "
			"<BR>The name of the sound has no other effect than making it easier "
			"work with the animation.</p>");
	QWhatsThis::add( changeNameButton, infoText );
	QObject::connect( changeNameButton, SIGNAL( clicked() ), 
			soundHandler, SLOT( setSoundName() ) );
	
	QLabel *soundsLabel = new QLabel( tr("Sounds:"), preferencesMenu );
	grid->addWidget( soundsLabel, 0, 1 );
	
	QListBox *soundsList = new QListBox(preferencesMenu);
	grid->addMultiCellWidget( soundsList, 1, 3, 1, 1, 0 );
	infoText = 
			tr("<h4>Sounds</h4> "
			"<p>This lists shows all the sounds connected to this frame.</p>"
			"<p>The sounds will begin playing when this frame is shown and play "
			"until they are done.</p>");
	QWhatsThis::add( soundsLabel, infoText );
	QWhatsThis::add( soundsList, infoText );
	
	
	
	QSpacerItem *space = new QSpacerItem(0, 7);
	grid->addItem( space, 4, 0 );
	
	preferencesMenu->setSoundsList(soundsList);
	soundHandler->setSoundsList(soundsList);
	
	/*
	QLabel *subTitleLabel = new QLabel(tr("Subtitle:"), preferencesMenu);
	subTitleLabel->setMaximumWidth(subTitleLabel->width());
	
	FlexibleLineEdit *subTitleInput = new FlexibleLineEdit(preferencesMenu);
	QObject::connect(subTitleInput, SIGNAL( lineEditCanceled() ),
			preferencesMenu, SLOT( close() ) );
	preferencesMenu->setFocusWidget(subTitleInput);
	QString subTitleText = 
			tr("<h4>Set subtitle</h4> "
			"<p>Here you can set the subtitle of the current frame/frames"
			"</p>");
    QWhatsThis::add( subTitleLabel, subTitleText );
	QWhatsThis::add( subTitleInput, subTitleText );
	*/
	frameBar->setPreferencesMenu(preferencesMenu);
	preferencesMenu->hide();
}


void MainWindowGUI::makeGotoMenu( QWidget * parent )
{
	gotoMenu = new MenuFrame( parent );
	editMenuHandler->setGotoMenu(gotoMenu);
	QGridLayout *grid = new QGridLayout( gotoMenu, 0, 2, 2 );

	QString gotoMenuText = 
			tr("<h4>Go to frame menu</h4> "
			"<p>Here you can specify a framenumber and the program will jump "
			"to the specified frame</p> ");
    QWhatsThis::add( gotoMenu, gotoMenuText );
	
	QLabel *gotoFrameLabel = new QLabel(tr("Go to frame:"), gotoMenu);
	gotoFrameLabel->setMaximumWidth(gotoFrameLabel->width());
	grid->addWidget( gotoFrameLabel, 0, 0 );
	
	gotoSpinner = new FlexibleSpinBox(gotoMenu);
	gotoSpinner->setMaximumWidth(60);
	gotoSpinner->setMinValue(1);
	gotoSpinner->setEnabled(false);
	grid->addWidget( gotoSpinner, 0, 1 );
	QObject::connect( frameBar, SIGNAL( newActiveFrame(int) ), 
			gotoSpinner, SLOT( setValue(int) ) );
	QObject::connect( frameBar, SIGNAL( modelSizeChanged(int) ),
			gotoSpinner, SLOT( setMaximumValue(int) ) );
	gotoMenu->hide();
	QObject::connect( gotoSpinner, SIGNAL( spinBoxTriggered(int) ), 
	        editMenuHandler, SLOT( gotoFrame(int) ) );
	QObject::connect( gotoSpinner, SIGNAL( spinBoxCanceled() ),
			editMenuHandler, SLOT( closeGotoMenu() ) );
	gotoMenu->setFocusWidget(gotoSpinner);
	
	
	QSpacerItem *space = new QSpacerItem(0, 0);
	grid->addItem( space, 0, 2 );
}


void MainWindowGUI::makeStatusBar()
{
	frameNumberDisplay = new QLabel(tr("Frame number: 0") ,this);
	this->statusBar()->addWidget(frameNumberDisplay, 0, true);
	QObject::connect( frameBar, SIGNAL(newActiveFrame(const QString &)), 
			frameNumberDisplay, SLOT(setText(const QString &)) );
	QString frameNumberDisplayText = 
			tr("<h4>Frame number</h4><p>This area displays the number" 
			"of the currently selected frame</p>");
    QWhatsThis::add( frameNumberDisplay, frameNumberDisplayText );
}


void MainWindowGUI::makeViews(QWidget *parent)
{
	QObject::connect(cameraHandler, SIGNAL(capturedFrame()), 
			this, SLOT(activateMenuOptions()));
	
	frameViewStack = new QWidgetStack(parent);
	
	frameView = new FrameView(frameViewStack);
	frameView->setBackgroundMode(NoBackground);
	
	frameViewStack->raiseWidget(frameView);
	DomainFacade::getFacade()->attatch(frameView);
	
	frameViewStack->setMinimumSize(400, 300);
	frameViewStack->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	
	cameraHandler->setWidgetStack(frameViewStack);
	cameraHandler->setFrameView(frameView);
	
	
	//VideoView *videoView = new GstVideoView(frameViewStack);
	//videoView->setBackgroundMode(NoBackground);
	VideoView *videoView = frameView;
	
	
	QObject::connect( videoView, SIGNAL(cameraReady()),
	 		cameraHandler, SLOT(switchToVideoView()) );
	
	QString infoText = 
			tr("<h4>GstVideoView</h4><p> In this area you can see the "
	 		"the video from the camera so that you can "
			"take snapshots.</p>");
    QWhatsThis::add( videoView, infoText );
	
	cameraHandler->setVideoView(videoView);
	
	
	infoText = 
			tr("<h4>FrameView</h4><p> In this area you can see the "
	 		"selected frame. You can also play "
			"animations in this window by pressing the "
			"<b>Play</b> button.</p>");
    QWhatsThis::add( frameView, infoText );
}


void MainWindowGUI::newProject() 
{
	bool b = DomainFacade::getFacade()->isUnsavedChanges();
	if (b) {
		int save = QMessageBox::question(this,
			tr("Unsaved changes"),
			tr("There are unsaved changes. Do you want to save?"),
			tr("&Yes"), tr("&No"), // button 0, button 1, ...
			QString::null, 0, 1 );
		if (save == 0) { // user pressed button 0, which is 'yes'
			saveProject();
		}
	}
	DomainFacade::getFacade()->newProject();
	fileMenu->setItemEnabled(SAVE, false);
}


void MainWindowGUI::openProject()
{
	QString file = QFileDialog::getOpenFileName(lastVisitedDir, "Stopmotion (*.sto)", this);
	if ( !file.isNull() ) {
		DomainFacade::getFacade()->openProject( file.ascii() );
		fileMenu->setItemEnabled(SAVE_AS, true);
		fileMenu->setItemEnabled(SAVE, true);
	}
}


void MainWindowGUI::saveProjectAs()
{
	QString file = QFileDialog::getSaveFileName(lastVisitedDir, "Stopmotion (*.sto)",
			this, "Save file dialog", tr("Save File"));

	if ( !file.isNull() ) {
		DomainFacade::getFacade()->saveProject( file.ascii() );
		fileMenu->setItemEnabled(SAVE, true);
	}
}


void MainWindowGUI::saveProject()
{
	const char *file = DomainFacade::getFacade()->getProjectFile();
	if (file) {
		DomainFacade::getFacade()->saveProject(file);
	}
	else {
		saveProjectAs();
	}
}


void MainWindowGUI::dragEnterEvent ( QDragEnterEvent * event)
{
	Logger::get().logDebug("Drag entering application");
	event->accept( QUriDrag::canDecode(event) );
}


void MainWindowGUI::dropEvent(QDropEvent *event)
{
	Logger::get().logDebug("Drop inside the application");

	if( QUriDrag::canDecode(event) ) {
		QStringList fileNames;
		
		if( QUriDrag::decodeLocalFiles(event, fileNames) ) {
			Logger::get().logDebug("test");
			modelHandler->addFrames(fileNames);
		}
	}
}


/**
 * @todo change so that CTRL+A instead of just A selects all the frames.
 */
void MainWindowGUI::keyPressEvent( QKeyEvent *k )
{
	switch ( k->key() ) 
	{
		case Key_Right: case Key_X:
		{
			runAnimationHandler->selectNextFrame();
			break;
		}
		case Key_Left: case Key_Z:
		{
			runAnimationHandler->selectPreviousFrame();
			break;
		}
		case Key_Delete:
		{
			modelHandler->removeFrames();
			break;
		}
		case Key_Shift:
		{
			frameBar->setSelecting(true);
			break;
		}
		case Key_A:
		{
			if( DomainFacade::getFacade()->getModelSize() > 1) {
				DomainFacade::getFacade()->setActiveFrame(0);
				frameBar->setSelection( DomainFacade::getFacade()->getModelSize()-1 );
			}
			break;
		}
		case Key_C:
		{
			cameraHandler->toggleCamera();
			break;
		}
		case Key_Space:
		{
			cameraHandler->captureFrame();
			break;
		}
		default:
		{
			k->ignore();
			break;
		}
	}
}


void MainWindowGUI::keyReleaseEvent ( QKeyEvent * k )
{
	switch ( k->key() ) 
	{
		case Key_Shift:
		{
			frameBar->setSelecting(false);
			break;
		}
	}
}


/**
 * @todo Refactor to class
 */
void MainWindowGUI::showAboutDialog()
{
	QTabDialog *aboutDialog = new QTabDialog(this);
	
	QLabel *aboutText = 
			new QLabel(tr(
						"<p>This is the stopmotion application for "
						"creating stopmotion animations.</p>"
						"<p>(c) 2005, Fredrik Berg Kjlstad and Bjrn Erik Nilsen</p>"), 
						aboutDialog);
	aboutDialog->addTab(aboutText, tr("&About"));
	
	QLabel *authorsText = 
			new QLabel(	"<p>Fredrik Berg Kjlstad <BR>fredrikbk@hotmail.com</p> "
						"<p>Bjrn Erik Nilsen     <BR>bjoern_erik_nilsen@hotmail.com</p> ", 
						aboutDialog );
	aboutDialog->addTab(authorsText, tr("A&uthors"));
	
	QLabel *thanksToText =
			new QLabel( "<p>Herman Robak <BR>herman@skolelinux.no</p> "
						"<p>yvind Kols <BR>pippin@gimp.org</p> ", aboutDialog );
	aboutDialog->addTab(thanksToText, tr("&Thanks To"));
	
	QTextEdit *licenceText = new QTextEdit( aboutDialog );
	
	licenceText->setReadOnly(true);
	licenceText->setHScrollBarMode( QScrollView::AlwaysOff );
	licenceText->setMinimumWidth(620);
	licenceText->setMinimumHeight(250);
	
	licenceText->setText( licence );
	
	aboutDialog->addTab(licenceText, tr("&Licence Agreement"));
	
	aboutDialog->show();
}


void MainWindowGUI::showPreferencesMenu()
{
	PreferencesMenu *preferencesMenu = new PreferencesMenu(this);
	preferencesMenu->show();
}


void MainWindowGUI::mousePressEvent( QMouseEvent * )
{
	preferencesMenu->close();
	gotoMenu->close(this);
	
}


void MainWindowGUI::modelSizeChanged( int modelSize )
{
	if(modelSize == 0) {
		editMenu->setItemEnabled(CUT, false);
		editMenu->setItemEnabled(COPY, false);
		editMenu->setItemEnabled(GOTO, false);
	}
}


void MainWindowGUI::activateMenuOptions()
{
	editMenu->setItemEnabled(UNDO, true);
	editMenu->setItemEnabled(REDO, true);
	editMenu->setItemEnabled(GOTO, true);
// 	editMenu->setItemEnabled(COPY, true);
	fileMenu->setItemEnabled(SAVE_AS, true);
}
