/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "preferencesmenu.h"

#include <qframe.h>
#include <qlayout.h>
#include <qlabel.h>

#include "src/foundation/preferencestool.h"
#include "flexiblelineedit.h"

#include <iostream>
using namespace std;

PreferencesMenu::PreferencesMenu( QWidget *parent ) : QTabDialog(parent)
{
	connectButtons();
	makeVideoImportTab();
	initializeImportValues();
	makeVideoExportTab();
	
	this->setApplyButton();
	this->setCancelButton();
	this->setHelpButton();
}


void PreferencesMenu::connectButtons()
{
	QObject::connect( this, SIGNAL(applyButtonPressed()), 
			this, SLOT(apply()) );
}


void PreferencesMenu::makeVideoImportTab()
{
	QFrame *importVideoTab = new QFrame(this);
	QGridLayout *grid = new QGridLayout( importVideoTab, 1, 1, 3 );
	
	QLabel *informationText = 
			new QLabel(tr(
				"<p>Below you can set which program/process stopmotion should use "
				"for grabbing images from the webcam, and displaying video.<br> "
				"<br> "
				"You can also add new programs, just make sure the command line "
				"is written with file path being <b>&lt;DEFAULTPATH&gt;</b></p>"), 
				importVideoTab);
	informationText->setMinimumWidth(440);
	informationText->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	
	
	deviceSelectionTable = new QTable(importVideoTab);
	deviceSelectionTable->setNumCols(2);
	deviceSelectionTable->horizontalHeader()->setLabel( 0, tr( "Active" ) );
	deviceSelectionTable->horizontalHeader()->setLabel( 1, tr( "Command line option" ) );
	deviceSelectionTable->adjustColumn(0);
	deviceSelectionTable->adjustColumn(1);
// 	QObject::connect( deviceSelectionTable, SIGNAL(currentChanged(int, int)),
// 			this, SLOT(activeRowChanged(int)) );
	QObject::connect( deviceSelectionTable, SIGNAL(valueChanged(int, int)),
			this, SLOT(valueChanged(int, int)) );
	
	QPushButton *removeButton = new QPushButton(tr("&Remove"), importVideoTab);
	
	inputLineEdit = new FlexibleLineEdit(importVideoTab);
 	QObject::connect( inputLineEdit, SIGNAL(returnPressed()), 
			this, SLOT(addImportProgram()) );
	
	
	addButton = new QPushButton(tr("&Add") ,importVideoTab);
	addButton->setFocusPolicy( QWidget::NoFocus );
	QObject::connect(addButton, SIGNAL(clicked()), 
			this, SLOT(addImportProgram()) );
	
	QSpacerItem *space3 = new QSpacerItem(0, 10, QSizePolicy::Fixed, QSizePolicy::Fixed);
	
	QSpacerItem *rightSpace = new QSpacerItem(5, 0, QSizePolicy::Fixed, QSizePolicy::Fixed);
	
	QSpacerItem *leftSpace = new QSpacerItem(5, 0, QSizePolicy::Fixed, QSizePolicy::Fixed);
	
	grid->addWidget( informationText, 0, 1 );
	grid->addMultiCellWidget( deviceSelectionTable, 1, 3, 1, 1 );
	grid->addWidget( removeButton, 3, 2 );
	grid->addWidget( inputLineEdit, 4, 1 );
	grid->addWidget( addButton, 4, 2 );
	grid->addItem( space3, 5, 1 );
	grid->addItem( rightSpace, 4, 3 );
	grid->addItem( leftSpace, 4, 0);
	
	
	importVideoTab->setMinimumHeight(300);
	
	this->addTab(importVideoTab, tr("Video &Import"));
}


void PreferencesMenu::makeVideoExportTab()
{
	QLabel *aboutText = 
			new QLabel(tr(
						"<p>Video Export</p>"), 
						this);
	this->addTab(aboutText, tr("Video &Export"));
}


void PreferencesMenu::addImportProgram()
{
	if(inputLineEdit->text() != "" ) {
		//deviceSelectionBox->insertItem(inputLineEdit->text());
		int newRow = deviceSelectionTable->numRows();
		deviceSelectionTable->insertRows(newRow);
		deviceSelectionTable->setItem( newRow, 0, 
				new QCheckTableItem( deviceSelectionTable, "") );
		deviceSelectionTable->setText( newRow, 1, inputLineEdit->text() );
		
		inputLineEdit->clear();
		inputLineEdit->clearFocus();
	}
}


void PreferencesMenu::activeRowChanged(int row)
{
	inputLineEdit->setText(deviceSelectionTable->text(row, 1));
}


void PreferencesMenu::valueChanged(int row, int column)
{
	switch (column)
	{
		case 0:
		{
			int size = deviceSelectionTable->numRows();
			for(int i = 0; i<size; i++) {
				if(i != row) {
					((QCheckTableItem*)deviceSelectionTable->item(i, 0))->
							setChecked(false);
				}
			}
			break;
		}
		case 1:
		{
			activeRowChanged(row);
			break;
		}
	}
}


void PreferencesMenu::initializeImportValues()
{
	QString defaultValues[NUMBER_OF_DEFAULTS] 
			= {
			"vgrabbj -f (DEFAULTPATH) "
			"-d /dev/video0 -b -i vga"
			};
	
	PreferencesTool* pref = PreferencesTool::get();
	const char* tmp;
	
	int numImports = pref->getPreference("numberofimports", 1);
	
	for(int i=0; i<numImports; i++) {
		if(i < NUMBER_OF_DEFAULTS) {
			tmp = pref->getPreference(QString("import%1").arg(i).ascii(), defaultValues[i]);
		}
		else {
			tmp = pref->getPreference(QString("import%1").arg(i).ascii(), "");
		}
		
		if(tmp != NULL) {
			int newRow = deviceSelectionTable->numRows();
			deviceSelectionTable->insertRows(newRow);
			deviceSelectionTable->setItem( newRow, 0, 
					new QCheckTableItem( deviceSelectionTable, "") );
			deviceSelectionTable->setText( newRow, 1, QString(tmp) );
		}
	}
	deviceSelectionTable->adjustColumn(1);
	
	int activeCommand = pref->getPreference("activecommandlineoption", 0);
	if(activeCommand > -1) {
		((QCheckTableItem*)deviceSelectionTable->item(activeCommand, 0))->setChecked(true);
	}
}


void PreferencesMenu::apply()
{
	PreferencesTool *pref = PreferencesTool::get();
	
	int size = deviceSelectionTable->numRows();
	int activeCommand = -1;
	pref->setPreference("numberofimports", size, true);

	for(int i=0; i<size; i++) {
		pref->setPreference(QString("import%1").arg(i), 
				deviceSelectionTable->text(i, 1).ascii(), true);
		
		if( ((QCheckTableItem*)deviceSelectionTable->item(i, 0))->isChecked() ) {
			activeCommand = i;
		}
	}

	pref->setPreference("activecommandlineoption", activeCommand, true);
	
	pref->flushPreferences();
}
