/***************************************************************************
 *   Copyright (C) 2005 by fredrik                                         *
 *   fredrik@Elanor                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "scenethumbview.h"

#include "graphics/icons/clapper.xpm"
#include "src/domain/domainfacade.h"

#include <qpainter.h>
#include <qimage.h>
#include <qapplication.h>
#include <qdragobject.h> 
#include <sstream>
#include <qpushbutton.h>
#include <qiconset.h>

#include <iostream>


SceneThumbView::SceneThumbView(FrameBar *frameBar, QWidget *parent, int number) 
 : ThumbView(frameBar, parent, number)
{
	this->setAcceptDrops(true);
	this->isOpened = false;
	
	arrowButton = new SceneArrowButton(this);
	int width = this->width();
	arrowButton->setGeometry( width-width/4, width/9, 16, 16 );
	QObject::connect( arrowButton, SIGNAL(clicked()), 
		this, SLOT(closeScene()) );
}


SceneThumbView::~SceneThumbView()
{
}


void SceneThumbView::setOpened( bool isOpened )
{
	this->isOpened = isOpened;
	arrowButton->setOpened(isOpened);
	this->repaint();
}


bool SceneThumbView::getIsOpened( )
{
	return isOpened;
}


/**
 * @todo the width can be cached somewhere so that the function width() don't have
 * to be called for every frame and scene thumbview.
 */
void SceneThumbView::paintEvent ( QPaintEvent * )
{
	int width = this->width();

	QPainter paint( this );
	paint.setPen( Qt::black );
	stringstream ss;
	ss << "Scene " << number+1;
	paint.drawText( 7, (width/4), ss.str() );
	
	QPixmap centerIcon;
	if(isOpened) {
		centerIcon = QPixmap(clapper);
		paint.drawPixmap(width/2-16, width/2-10, centerIcon);
	}
	else {
		if(DomainFacade::getFacade()->getSceneSize(number) > 0) {
			centerIcon = QPixmap( QImage(
					DomainFacade::getFacade()->getFrame(0, number)->getImagePath()).
					scale(50, 50) );
			paint.drawPixmap(width/2-25, width/2-16, centerIcon);
		}
		else {
			centerIcon = QPixmap(clapper);
			paint.drawPixmap(width/2-16, width/2-10, centerIcon);
		}
	}
}


void SceneThumbView::mousePressEvent(QMouseEvent *e)
{
	//For calculating the manhattenLength to avoid unvanted
	//drags.
	dragPos = e->pos();
}


void SceneThumbView::mouseReleaseEvent( QMouseEvent * )
{
	if(DomainFacade::getFacade()->getActiveSceneNumber() != this->number) {
		DomainFacade::getFacade()->setActiveScene(number);
	}
}


void SceneThumbView::mouseMoveEvent(QMouseEvent * me)
{
	if(me->state() & LeftButton) {
		int distance = (me->pos() - dragPos).manhattanLength();
		if(distance > QApplication::startDragDistance()) {
			startDrag();
		}
	}
	QLabel::mouseMoveEvent(me);
}


void SceneThumbView::dropEvent(QDropEvent * event)
{
	int movingScene = frameBar->getMovingScene();
	if( (event->source() != 0) && (movingScene != this->number) && 
			(movingScene != -1) ) {
		
		Logger::get().logDebug("Moving scene");
		
		DomainFacade::getFacade()->moveScene(movingScene, this->number);
	}
}


void SceneThumbView::startDrag()
{
	Logger::get().logDebug("Starting drag of the scene");
	
	frameBar->setMovingScene(this->number);
	
	QStrList lst;
	
	/*unsigned int size = DomainFacade::getFacade()->getSceneSize( 
			DomainFacade::getFacade()->getActiveSceneNumber() );
	for(unsigned int i=0; i<size; i++) {
		lst.append( DomainFacade::getFacade()->getFrame(i)->getImagePath() );
	}*/
	
	QUriDrag *drag = new QUriDrag( lst, this );
	drag->drag();
}


void SceneThumbView::dragEnterEvent( QDragEnterEvent * event )
{
	Logger::get().logDebug("Drag entering the SceneThumbView");
	event->accept( QUriDrag::canDecode(event) );
}


void SceneThumbView::closeScene()
{
	if(DomainFacade::getFacade()->getActiveSceneNumber() == this->number) {
		DomainFacade::getFacade()->setActiveScene(-1);
		cout << "closing" << endl;
	}
	else {
		DomainFacade::getFacade()->setActiveScene(number);
	}
}
