/***************************************************************************
 *   Copyright (C) 2005 by fredrik                                         *
 *   fredrik@Elanor                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "toolsmenu.h"

#include "graphics/icons/play.xpm"
#include "graphics/icons/nextframe.xpm"
#include "graphics/icons/previousframe.xpm"
#include "graphics/icons/steptoend.xpm"
#include "graphics/icons/steptobeginning.xpm"
#include "graphics/icons/stop.xpm"
#include "graphics/icons/pause.xpm"
#include "graphics/icons/loop.xpm"
#include "graphics/icons/capture.xpm"
#include "graphics/icons/cameraon.xpm"

#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qslider.h>


ToolsMenu::ToolsMenu( RunAnimationHandler *runAnimationHandler, ModelHandler *modelHandler,
		CameraHandler *cameraHandler, QWidget *parent ) : QFrame(parent),
		runAnimationHandler(runAnimationHandler), modelHandler(modelHandler), 
		cameraHandler(cameraHandler)
{
	addWidgets();
}


void ToolsMenu::addWidgets( )
{
	grid = new QGridLayout( this, 1, 1, 3 );
	
	
	//Create widgets for the toolsmenu:
	QPushButton *addFrameButton = new QPushButton( tr("&Add Frames"), this );
	addFrameButton->setFocusPolicy( QWidget::NoFocus );
	QString infoText = 
			tr("<h4>Add Frames</h4> "
			"<p>Click on this button to <em>add</em> frames to the "
			"animation.</p>");
	QWhatsThis::add(addFrameButton, infoText);
	QObject::connect(addFrameButton, SIGNAL(clicked()), modelHandler, SLOT(chooseFrame()));
	
	
	QPushButton *removeFramesButton = new QPushButton( tr("&Remove Selection"), this );
	removeFramesButton->setFocusPolicy( QWidget::NoFocus );
	infoText = 
			tr("<h4>Remove Selection</h4> "
			"<p>Click this button to <em>remove</em> the selected frames "
			"from the animation.</p>");
	QWhatsThis::add(removeFramesButton, infoText);
	QObject::connect(removeFramesButton, SIGNAL(clicked()), modelHandler, SLOT(removeFrames()));
	runAnimationHandler->setRemoveFramesButton(removeFramesButton);
	modelHandler->setRemoveFramesButton(removeFramesButton);
	
	
	QPushButton *newSceneButton = new QPushButton( tr("New &Scene"), this );
	newSceneButton->setFocusPolicy( QWidget::NoFocus );
	infoText = 
			tr("<h4>New Scene</h4> "
			"<p>Click this button to <em>create</em> a new <em>scene</em> "
			"to the animation.</p>");
	QWhatsThis::add(newSceneButton, infoText);
	QObject::connect(newSceneButton, SIGNAL(clicked()), modelHandler, SLOT(newScene()));
	
	
	QPushButton *removeSceneButton = new QPushButton( tr("Re&move Scene"), this);
	removeSceneButton->setFocusPolicy( QWidget::NoFocus );
	infoText = 
			tr("<h4>Remove Scene</h4> "
			"<p>Click this button to <em>remove</em> the selected scene "
			"from the animation.</p>");
	QWhatsThis::add(removeSceneButton, infoText);
	QObject::connect(removeSceneButton, SIGNAL(clicked()), modelHandler, SLOT(removeScene()));
	
	
	QPushButton *cameraButton = new QPushButton( this );
	cameraButton->setPixmap( QPixmap(cameraon) );
	cameraButton->setFocusPolicy( QWidget::NoFocus );	
	cameraHandler->setCameraButton(cameraButton);
	infoText = 
			tr("<h4>Toggle camera on/off</h4> "
			"<p>Click this button to toggle the camera on and off</p> "
			"<p>You can also toggle the camera by pressing the <b>C key</b></p>");
	QWhatsThis::add( cameraButton, infoText );
	QToolTip::add( cameraButton, infoText );
	QObject::connect( cameraButton, SIGNAL(clicked()), 
			cameraHandler, SLOT(toggleCamera()) );
	
	
	captureGroupSpacer = new QSpacerItem(0, 20, QSizePolicy::Fixed, QSizePolicy::Fixed);
	
	
	//Groupbox containing buttons related to capturing frame from the camera.
	captureGroup = new QGroupBox(this);
	captureGroup->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
	captureGroup->hide();
	QObject::connect( cameraHandler, SIGNAL(cameraStateChanged(bool)), 
			this, SLOT(activateCaptureGroup(bool)) );
	
	QGridLayout *captureGrid = new QGridLayout( captureGroup, 1, 1, 1 );
	
	
	QSpacerItem *captureGroupSpace1 = new QSpacerItem(5, 0, QSizePolicy::Fixed, 
			QSizePolicy::Fixed);
	QSpacerItem *captureGroupSpace2 = new QSpacerItem(5, 0, QSizePolicy::Fixed, 
			QSizePolicy::Fixed);
	QSpacerItem *captureGroupSpace3 = new QSpacerItem(0, 8, QSizePolicy::Fixed, 
			QSizePolicy::Fixed);
	QSpacerItem *captureGroupSpace4 = new QSpacerItem(0, 5, QSizePolicy::Fixed, 
			QSizePolicy::Fixed);
	
	
	QPushButton *captureFrameButton = new QPushButton( captureGroup );
	captureFrameButton->setPixmap( QPixmap(captureicon) );
	captureFrameButton->setFocusPolicy( QWidget::NoFocus );
	infoText =
			tr("<h4>Capture Frame</h4> "
			"<p>Click on this button to <em>capture</em> a frame from the "
			"camera an put it in the animation</p> <p> This can also be "
			"done by pressing the <b>Space key</b></p>");
	QWhatsThis::add( captureFrameButton, infoText );
	QToolTip::add( captureFrameButton, infoText );
	QObject::connect(captureFrameButton, SIGNAL(clicked()), 
			cameraHandler, SLOT(captureFrame()));
	
	
	QSpacerItem *mixSliderSpace = new QSpacerItem(0, 7, QSizePolicy::Fixed, QSizePolicy::Fixed);

	
	QLabel *mixSliderCaption = new QLabel( tr("Number of mixes:"), captureGroup );
	mixSliderCaption->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	
	QSlider *mixSlider = new QSlider(0, 5, 1, 2, Qt::Horizontal, captureGroup);
	mixSlider->setTickmarks(QSlider::Below);
	mixSlider->setFocusPolicy( QWidget::NoFocus );
	infoText = 
			tr("<h4>Number of mixes</h4> "
			"<p>By changing the value in this slidebar you can specify how many images "
			"backwards in the animation which should be mixed on top of the camera. </p> "
			"<p>By mixing the previous image(s) onto the camera you can more easily see "
			"how the next shot will be in relation to the other, therby making a smoother "
			"stop motion animation!</p>");
	QWhatsThis::add( mixSliderCaption, infoText );
	QWhatsThis::add( mixSlider, infoText );
	QObject::connect( mixSlider, SIGNAL(valueChanged(int)), 
			cameraHandler, SLOT(setMixCount(int)) );
	
	//Groupbox containing buttons related to running the animation.
	QGroupBox *runAnimationGroup = new QGroupBox(this);
	runAnimationGroup->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	QGridLayout *runAnimationGrid = new QGridLayout( runAnimationGroup, 1, 1, 1 );
	
	QSpacerItem *space4 = new QSpacerItem(5, 0, QSizePolicy::Fixed, QSizePolicy::Fixed);
	QSpacerItem *space5 = new QSpacerItem(5, 0, QSizePolicy::Fixed, QSizePolicy::Fixed);
	QSpacerItem *space6 = new QSpacerItem(0, 8, QSizePolicy::Fixed, QSizePolicy::Fixed);
	QSpacerItem *space7 = new QSpacerItem(0, 5, QSizePolicy::Fixed, QSizePolicy::Fixed);

	
	QSpacerItem *space1 = new QSpacerItem(0, 7, QSizePolicy::Fixed, QSizePolicy::Fixed);

	
	QLabel *animationSpeedChooserCaption = new QLabel( tr("FPS chooser"), runAnimationGroup );
	animationSpeedChooserCaption->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, 
			QSizePolicy::Fixed));
	
	
	QSpinBox *animationSpeedChooser = new QSpinBox( 1, 30, 1, runAnimationGroup );
	animationSpeedChooser->setFocusPolicy( QWidget::NoFocus );
	animationSpeedChooser->setValue(10);
	infoText = 
			tr("<h4>FPS chooser</h4> "
			"<p>By changing the value in this "
			"chooser you set which speed the "
			"animation in the <b>FrameView</b> "
			"should run at.</p> "
			"<p>To start an animation press the "
			"<b>Run Animation</b> button.</p>");
    QWhatsThis::add( animationSpeedChooserCaption, infoText );
    QWhatsThis::add( animationSpeedChooser, infoText );
	QObject::connect( animationSpeedChooser, SIGNAL( valueChanged(int) ),
			runAnimationHandler, SLOT(setSpeed(int)) );
	
	
	QSpacerItem *space2 = new QSpacerItem(0, 25, QSizePolicy::Fixed, QSizePolicy::Fixed);
	
	
	QPushButton *playButton = new QPushButton( runAnimationGroup );
	playButton->setIconSet( QIconSet(QPixmap(playicon)) );
	playButton->setFocusPolicy( QWidget::NoFocus );
	runAnimationHandler->setPlayButton(playButton);
	infoText =
			tr("<h4>Play animation</h4>");
	QToolTip::add(playButton, infoText);
	QObject::connect(playButton, SIGNAL(clicked()), runAnimationHandler, 
			SLOT(runAnimation()));
	
	
	QPushButton *nextFrameButton = new QPushButton( runAnimationGroup );
	nextFrameButton->setIconSet( QIconSet(QPixmap(nextframeicon)) );
	nextFrameButton->setFocusPolicy( QWidget::NoFocus );
	nextFrameButton->setAutoRepeat(true);
	QObject::connect( nextFrameButton, SIGNAL(clicked()), 
			runAnimationHandler, SLOT(selectNextFrame()) );
	
	
	QPushButton *previousFrameButton = new QPushButton( runAnimationGroup );
	previousFrameButton->setIconSet( QIconSet(QPixmap(previousframeicon)) );
	previousFrameButton->setFocusPolicy( QWidget::NoFocus );
	previousFrameButton->setAutoRepeat(true);
	QObject::connect( previousFrameButton, SIGNAL(clicked()), 
			runAnimationHandler, SLOT(selectPreviousFrame()) );
	
	
	QPushButton *toEndButton = new QPushButton( runAnimationGroup );
	toEndButton->setIconSet( QIconSet(QPixmap(steptoendicon)) );
	toEndButton->setFocusPolicy( QWidget::NoFocus );
	toEndButton->setEnabled(false);
	
	
	QPushButton *toBeginningButton = new QPushButton( runAnimationGroup );
	toBeginningButton->setIconSet( QIconSet(QPixmap(steptobeginningicon)) );
	toBeginningButton->setFocusPolicy( QWidget::NoFocus );
	toBeginningButton->setEnabled(false);
	
	
	QPushButton *stopButton = new QPushButton( runAnimationGroup );
	stopButton->setIconSet( QIconSet(QPixmap(stopicon)) );
	stopButton->setFocusPolicy( QWidget::NoFocus );
	infoText =
			tr("<h4>Stop animation</h4>");
	QToolTip::add(stopButton, infoText);
	QObject::connect( stopButton, SIGNAL(clicked()), 
			runAnimationHandler, SLOT(stopAnimation()) );

	
	QPushButton *pauseButton = new QPushButton( runAnimationGroup );
	pauseButton->setIconSet( QIconSet(QPixmap(pauseicon)) );
	pauseButton->setFocusPolicy( QWidget::NoFocus );
	pauseButton->setEnabled(false);
	runAnimationHandler->setPauseButton(pauseButton);
	QObject::connect(pauseButton, SIGNAL(clicked()), 
			runAnimationHandler, SLOT(pauseAnimation()) );
	
	
	QPushButton *loopButton = new QPushButton( runAnimationGroup );
	loopButton->setIconSet( QIconSet(QPixmap(loopicon)) );
	loopButton->setFocusPolicy( QWidget::NoFocus );
	loopButton->setToggleButton(true);
	//loopButton->setEnabled(false);
	runAnimationHandler->setLoopButton(loopButton);
	infoText =
			tr("<h4>Loop animation</h4> <p>With this button you can set whether "
			"you want the animation to play to the end, or to loop indefinetly.</p>");
	QToolTip::add(loopButton, infoText);
	QObject::connect( loopButton, SIGNAL(clicked()), 
			runAnimationHandler, SLOT(toggleLooping()) );
	
	
	QSpacerItem *space3 = new QSpacerItem(0, 0);
	
	
	//Add the widgets to the grid
	grid->addMultiCellWidget(addFrameButton, 0, 0, 0, 1);
	grid->addMultiCellWidget(removeFramesButton, 1, 1, 0, 1);
	grid->addMultiCellWidget(newSceneButton, 2, 2, 0, 1);
	grid->addMultiCellWidget(removeSceneButton, 3, 3, 0, 1);
	grid->addMultiCellWidget(cameraButton, 4, 4, 0, 1);
 	grid->addItem( captureGroupSpacer, 5, 0 );
	
	captureGrid->addWidget( captureFrameButton, 1, 1 );
	captureGrid->addItem( mixSliderSpace, 2, 1 );
	captureGrid->addMultiCellWidget( mixSliderCaption, 3, 3, 1, 2 );
	captureGrid->addMultiCellWidget( mixSlider, 4, 4, 1, 2 );
	captureGrid->addItem( captureGroupSpace1, 1, 0 );
	captureGrid->addItem( captureGroupSpace2, 1, 3 );
	captureGrid->addItem( captureGroupSpace3, 0, 1 );
	captureGrid->addItem( captureGroupSpace4, 5, 1 ); 
	
	grid->addMultiCellWidget(captureGroup, 6, 6, 0, 1);
	grid->addItem( space2, 7, 0 );
	
	runAnimationGrid->addWidget( toBeginningButton, 1, 1 );
	runAnimationGrid->addWidget( previousFrameButton, 1, 2 );
	runAnimationGrid->addWidget( playButton, 1, 3 );
	runAnimationGrid->addWidget( nextFrameButton, 1, 4 );
	runAnimationGrid->addWidget( toEndButton, 1, 5 );
	runAnimationGrid->addWidget( stopButton, 2, 3 );
	runAnimationGrid->addWidget( pauseButton, 2, 2 );
	runAnimationGrid->addWidget( loopButton, 2, 4 );
	runAnimationGrid->addItem( space1, 3, 1 );
	runAnimationGrid->addMultiCellWidget( animationSpeedChooserCaption, 4, 4, 1, 5 );
	runAnimationGrid->addMultiCellWidget( animationSpeedChooser, 5, 5, 1, 5 );
	
	runAnimationGrid->addItem( space4, 1, 0 );
	runAnimationGrid->addItem( space5, 1, 6 );
	runAnimationGrid->addItem( space6, 0, 1 );
	runAnimationGrid->addItem( space7, 6, 1 ); 
	
	grid->addMultiCellWidget(runAnimationGroup, 8, 8, 0, 1 );
	grid->addItem( space3, 9, 0 );
}


void ToolsMenu::activateCaptureGroup(bool activate)
{
	if(activate) {		
		captureGroup->show();
	}
	else {
		captureGroup->hide();
	}
}
